
* PROGRAM: COST_SHARING_ASC.SAS;
* DESCRIPTION: PERFORM COST SHARING ALGORITHM FOR ASC CLAIMS;

DATA ASC_BASEID;
RETAIN JOIN_ID '1';
SET &INC(WHERE=(CATEGORY='9b'));
RUN;

data ASC_FILE;
    SET  PBP_FILE (KEEP =
                   CONTRACT_ID PLAN_ID SEGMENT_ID HPLAN
                   PBP_B9B_DED_AMT PBP_B9B_MAXENR_AMT
                   PBP_B9B_MAXENR_PER PBP_B9B_COINS_PCT_MC PBP_B9B_COPAY_MC_AMT
                   PBP_B9B_MAXENR_TYPE PBP_B9A_MAXENR_AMT PBP_B9A_MAXENR_PER);
  join_id = '1';
run;

proc sql;
  create table cat_join as
      (select b.category, b.baseid, b.visits, b.amttot, b.year, p.*
      from ASC_BASEID b, ASC_FILE p
      where b.join_id = p.join_id)
    ;
quit;
run;

%MACRO ASC_COST_SHARE(SERV_CAT);

%LET SERV_CAT = ASC;
  DATA &SERV_CAT;
    set cat_join (drop = join_id);
    COST_SHARE_COST = 0;
    OTHER_COST = 0;

    IF PBP_B9B_MAXENR_AMT = . AND PBP_B9B_MAXENR_TYPE = '1' THEN DO;
      MAXENR_AMT = PBP_B9A_MAXENR_AMT;
      MAXENR_PER = PBP_B9A_MAXENR_PER;
    END;
    ELSE DO;
      MAXENR_AMT = PBP_B9B_MAXENR_AMT;
      MAXENR_PER = PBP_B9B_MAXENR_PER;
    END;

    IF MAXENR_AMT NE . THEN BENE_MAXIMUM = MAXENR_AMT;


      IF PBP_B9B_DED_AMT = . THEN DO;
        COST = SUM(COST,(VISITS*PBP_B9B_COPAY_MC_AMT));
        COST = SUM(COST,(AMTTOT*PBP_B9B_COINS_PCT_MC));
      END;
      ELSE DO;
        IF AMTTOT <= PBP_B9B_DED_AMT THEN DO;
          COST = AMTTOT;
        END;
        ELSE DO;
          COST = PBP_B9B_DED_AMT;
          DEDUCT_SHARE = PBP_B9B_DED_AMT / AMTTOT;
          AMTTOT = SUM(AMTTOT,-PBP_B9B_DED_AMT);
          VISITS = VISITS * (1 - DEDUCT_SHARE);
          COST = SUM(COST,(VISITS*PBP_B9B_COPAY_MC_AMT));
          COST = SUM(COST,(AMTTOT*PBP_B9B_COINS_PCT_MC));
        END;
      END;

    IF BENE_MAXIMUM >= 0 AND COST >= 0 THEN COST = MIN(COST,BENE_MAXIMUM);

    COST_SHARE_COST = COST;

    COST_SHARE_COST_MC  = COST;
    COST_SHARE_COST_AMO = 0;

    CATEGORY = "9b";
  RUN;

%MEND ASC_COST_SHARE;

%MACRO RUN_COST;
%LET CAT=ASC;
RUN;

%ASC_COST_SHARE(&CAT);

%MEND RUN_COST;

%RUN_COST;
RUN;

PROC APPEND DATA=ASC(KEEP=HPLAN YEAR BASEID COST_SHARE_COST COST_SHARE_COST_MC COST_SHARE_COST_AMO
       OTHER_COST COST CATEGORY AMTTOT)  BASE= COSTDATA FORCE;
RUN;

PROC DATASETS LIBRARY=WORK NODETAILS NOLIST;
DELETE ASC ASC_BASEID ASC_FILE CAT_JOIN;
QUIT;
RUN;
