
* PROGRAM: COST_SHARING_COMP_XRAY.SAS;
* DESCRIPTION: PERFORM COST SHARING ALGORITHM FOR COMP XRAY CLAIMS;

DATA COMPXRAY_BASEID;
RETAIN JOIN_ID '1';
SET &INC(WHERE=(CATEGORY='8bd'));
RUN;

data COMPXRAY_FILE;
    SET  PBP_FILE(KEEP =
                   CONTRACT_ID PLAN_ID SEGMENT_ID HPLAN
                   PBP_B8B_MAXENR_AMT PBP_B8B_MAXENR_PER
                   PBP_B8B_COPAY_AMT_DRS PBP_B8B_COINS_PCT_DRS
                   PBP_B8B_DED_AMT);
  join_id = '1';
run;

proc sql;
  create table cat_join as
      (select b.category, b.baseid, b.visits, b.amttot, b.year, p.*
      from COMPXRAY_BASEID b, COMPXRAY_FILE p
      where b.join_id = p.join_id)
    ;
quit;
run;

%MACRO COMPXRAY_COST_SHARE(SERV_CAT);

%LET SERV_CAT = COMPXRAY;
  DATA &SERV_CAT;
    set cat_join (drop = join_id);
    COST_SHARE_COST = 0;
    OTHER_COST = 0;

    IF PBP_B8B_MAXENR_AMT NE . THEN BENE_MAXIMUM = PBP_B8B_MAXENR_AMT;

    COST = .;

    COST = SUM(COST,(VISITS*PBP_B8B_COPAY_AMT_DRS));
    COST = SUM(COST,(AMTTOT*PBP_B8B_COINS_PCT_DRS));
    IF BENE_MAXIMUM >= 0 AND COST >= 0 THEN COST = MIN(COST,BENE_MAXIMUM);

    COST_SHARE_COST = COST;

    COST_SHARE_COST_MC  = COST;
    COST_SHARE_COST_AMO = 0;

    CATEGORY = "8bd";
  RUN;

%MEND COMPXRAY_COST_SHARE;

%MACRO RUN_COST;
%LET CAT=COMPXRAY;
RUN;
%COMPXRAY_COST_SHARE(&CAT);

%MEND RUN_COST;

%RUN_COST;
RUN;

PROC APPEND DATA=COMPXRAY(KEEP=HPLAN YEAR BASEID COST_SHARE_COST COST_SHARE_COST_MC COST_SHARE_COST_AMO
       OTHER_COST COST CATEGORY AMTTOT)  BASE= COSTDATA FORCE;
RUN;

PROC DATASETS LIBRARY=WORK NODETAILS NOLIST;
DELETE COMPXRAY COMPXRAY_BASEID COMPXRAY_FILE CAT_JOIN;
QUIT;
RUN;
